/********************************************************************
**	File:		ctidrain.c
**	Author:		Allan Smith
**			Connect Tech Inc.
**
**	Original:	March 28/94
**	Revision History:
**			May 30/94	Revised format slightly
**
**	Copyright (c) 1994 CTI, Connect Tech Inc. All Rights Reserved.
**	
**	THIS IS THE UNPUBLISHED PROPRIETARY SOURCE CODE OF CONNECT TECH INC.
**	The copyright notice above does not evidence any actual or intended
**	publication of such source code.
**	
**	This module contains Proprietary Information of Connect Tech, Inc
**	and should be treated as Confidential.
**
*********************************************************************
**
**	Description:
**	This program acts as replacement for the QNX 4.xx function called
**	tcdrain.  Being that the standard call does not properly support
**	buffered IO, a replacement had to be created.
**
**	The syntax of ctidrain is identical to Watcom's tcdrain function:
**
**	#include	<termios.h>
**	int	ctidrain( int fildes );
**
**	Return Values:
**	EBADF	The argument fildes is invalid.
**	EINTR	A signal interrupted the operation.
**	ENOSYS	The resource manager associated with fildes does not
**			support this call.
**	ENOTTY	The argument fildes does not refer to a terminal device.
**
********************************************************************/
#include	<sys/kernel.h>
#include	<sys/dev.h>
#include	<errno.h>
#include	"ctidrain.h"

int	ctidrain( int fildes ){
	struct _ctidrainmsg	msg;
	struct _dev_info_entry	info;

	/* dev_info call needed rather than just a Sendfd() since		*
	 * need to send the unit number to Dev.cti also 				*/		                         
	if( dev_info( fildes, &info ) != 0 )
		return( EBADF );
		
	msg.type = CTIFNS;			                         
	msg.subtype = CTIDRAIN;
	msg.port = info.unit - 1;
	return( Send( info.driver_pid, &msg, &msg, sizeof( struct _ctidrainmsg ),\
		sizeof( struct _ctidrainmsg ) )  );
}
