/********************************************************************
**	File:		ctiflush.c
**	Author:		CTI Enginneering
**			Connect Tech Inc.
**
**	Original:	Jan 19/96
**	Revision History:
**
**
*********************************************************************
**
**	Description:
**	This program acts as replacement for the QNX 4.xx function called
**	tcflush.  Being that the standard call does not properly support
**	buffered IO, a replacement had to be created.
**
**	The syntax of ctiflush is identical to Watcom's tcflush function:
**
**	#include	<termios.h>
**	int	ctiflush( int fildes, int queue_selector );
**
**	Return Values:
**	EBADF	The argument fildes is invalid.
**	EINVAL	The argument queue_selector is invalid.
**	ENOSYS	The resource manager associated with fildes does not
**			support this call.
**	ENOTTY	The argument fildes does not refer to a terminal device.
**
********************************************************************/
#include	<sys/kernel.h>
#include	<sys/dev.h>
#include	<sys/qioctl.h>
#include	<errno.h>
#include	<termios.h>
#include	"ctiflush.h"

int	ctiflush( int fildes, int queue_selector ){
	struct _dev_info_entry	info;

	/* dev_info call needed rather than just a Sendfd() since
	 * need to send the unit number to Dev.cti also 	*/		                         
	if( dev_info( fildes, &info ) != 0 )
		return( EBADF );
	
	/* if funtion to be performed is not one of these three then
	 * return error */
	if( queue_selector != TCIFLUSH && queue_selector != TCOFLUSH &&
	    queue_selector != TCIOFLUSH )
		return( EINVAL );
/* Flush Input*/	
	if( queue_selector == TCIFLUSH || queue_selector == TCIOFLUSH ){
		if( qnx_ioctl( fildes, CTIFLUSH_IN, (void *)0, 0, (void *)0, 0 ) == -1 )
			return( ENOSYS );
	}	
/* Flush output */
	if( queue_selector == TCOFLUSH || queue_selector == TCIOFLUSH ){
		if( qnx_ioctl( fildes, CTIFLUSH_OUT, (void *)0, 0, (void *)0, 0 ) == -1 )
			return( ENOSYS );
	}
	/* now perform a standard tcflush() to clear Dev's chars */
	return( tcflush( fildes, queue_selector ) );
}
