/**********************************************************************
**	File:			ptm.c
**	Author:			DJ Worthen
**	
**	Description:	Demo program to illustrate Packet Trigger Mode
**	Support routines. None
**	Revision history:
**	Original:		
**	
**	Copyright (c) 1995 CTI, Connect Tech Inc. All Rights Reserved.
**	
**	THIS IS THE UNPUBLISHED PROPRIETARY SOURCE CODE OF CONNECT TECH INC.
**	The copyright notice above does not evidence any actual or intended
**	publication of such source code.
**	
**	This module contains Proprietary Information of Connect Tech, Inc
**	and should be treated as Confidential.
**	
**	Packet Trigger mode allows you to define a Proxy that will
**	be Triggered each time a certain end-of-packet character is
**	seen in the input data stream. The end-of-packet character
**	is defined by the c_cc[VLEFT] member of the termios structure
**	for the channel.
**	
**	This program is a small demo that shows how you can read
**	packets from a channel while at the same time "listen" for
**	messages from other processes, without polling.
**	
**********************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/kernel.h>
#include <sys/types.h>
#include <sys/qioctl.h>
#include <sys/stat.h>
#include <sys/dev.h>
#include <sys/proxy.h>
#include <fcntl.h>
#include <termios.h>

struct termios termios_p;
unsigned char SendBuffer[8];
unsigned char Msg[80];
unsigned char DataBuff[256];
pid_t MyProxy;
int Status;
int NBytes;


main( int argc, char *argv[] ) 
{
	int	fd;
	int	i;
	pid_t	pid;

// Ensure the correct parms, then open the port
	if (argc != 2) {
		printf( "Usage: ptm port\n" );
		exit( -1 );
	}

	fd = open( argv[1], O_RDWR );
	if (fd < 0) {
		printf( "Could not open %s\n", argv[1] );
		exit( -1 );
	}

// Get the current termios parms for the channel
	if (tcgetattr( fd, &termios_p )) {
		printf( "Could not tcgetattr()\n" );
		exit( -1 );
	}

// Change only those that we need to
	termios_p.c_cc[ VMIN ] = 1;
	termios_p.c_cc[ VTIME ] = 0;
	termios_p.c_lflag &= ~(ECHO|ICANON|ISIG|ECHOE|ECHOK|ECHONL);
	termios_p.c_oflag&= ~(OPOST);
	termios_p.c_cc[ VLEFT ] = 0x03; //This is the match char
	if (tcsetattr( fd, TCSANOW, &termios_p )) {
		printf( "Could not tcsetattr()\n" );
		exit( -1 );
	}

// Attach to any proxy
	MyProxy = qnx_proxy_attach( 0, 0, 0, -1 );
	if (MyProxy == -1) {
		printf( "Unable to attach proxy\n" );
		exit( -1 );
	}

// Give the Trigger proxy to Dev.cti and enable Proxy Trigger Mode
// Note: request=0x300b turns on PTM, request=0x300c turns off PTM.
// Note: Driver expects to see 32 bits as proxy value, we do it this
//   way to ensure code works under 16 and 32 bit compiles.
	*((unsigned long *)&SendBuffer[0]) = 0;
	*((pid_t *)&SendBuffer[0]) = MyProxy;
	if ((Status = qnx_ioctl( fd, 0x300b, SendBuffer, sizeof( SendBuffer ),
		SendBuffer, sizeof( SendBuffer ))) < 0) {
		printf( "Unable to qnx_ioctl() (%d)\n", Status );
		exit( -1 );
	}

// Go into a loop reading "packets" from the line and messages
// from other processes.
	for ( ;; ) {
		pid = Receive( 0, Msg, sizeof( Msg ));
		if (pid == MyProxy) { // Message from Dev.cti
			NBytes = dev_read( fd, DataBuff, sizeof( DataBuff ),
				0, 0, 0, 0, 0 );
			printf( "%d Bytes received: ", NBytes );
			for(i = 0; i < NBytes; ++i ) 
				printf( "%02X ", DataBuff[i] );
			printf( "\n" );
		}
		else { // Message from somebody else
			printf( "Message from process %d: ", pid );
			for(i = 0; i < 10; ++i ) 
				printf( "%02X ", Msg[i] );
			printf( "\n" );
		}
	}
}

